package tools

import (
	"encoding/json"
	"fmt"
	"os"
	"strconv"
)

// SourceMaterial 原始JSON结构
type SourceMaterial struct {
	ID         int `json:"id"`
	Attributes struct {
		Name        string  `json:"name"`
		Desc        *string `json:"desc"`
		CreatedAt   string  `json:"createdAt"`
		UpdatedAt   string  `json:"updatedAt"`
		PublishedAt string  `json:"publishedAt"`
		Img         struct {
			Data struct {
				ID         int `json:"id"`
				Attributes struct {
					Name       string `json:"name"`
					Width      int    `json:"width"`
					Height     int    `json:"height"`
					URL        string `json:"url"`
					PreviewURL string `json:"previewUrl"`
					Formats    struct {
						Large     json.RawMessage `json:"large,omitempty"`
						Small     json.RawMessage `json:"small,omitempty"`
						Medium    json.RawMessage `json:"medium,omitempty"`
						Thumbnail json.RawMessage `json:"thumbnail,omitempty"`
					} `json:"formats"`
				} `json:"attributes"`
			} `json:"data"`
		} `json:"img"`
	} `json:"attributes"`
}

// TargetMaterial 目标JSON结构
type TargetMaterial struct {
	ID          int    `json:"id"`
	Name        string `json:"name"`
	Desc        string `json:"desc"`
	CreatedAt   string `json:"createdAt"`
	UpdatedAt   string `json:"updatedAt"`
	PublishedAt string `json:"publishedAt"`
	Img         struct {
		Data struct {
			ID         string `json:"id"`
			Name       string `json:"name"`
			Width      int    `json:"width"`
			Height     int    `json:"height"`
			URL        string `json:"url"`
			PreviewURL string `json:"previewUrl"`
			Formats    struct {
				Large     json.RawMessage `json:"large,omitempty"`
				Small     json.RawMessage `json:"small,omitempty"`
				Medium    json.RawMessage `json:"medium,omitempty"`
				Thumbnail json.RawMessage `json:"thumbnail,omitempty"`
			} `json:"formats"`
		} `json:"data"`
	} `json:"img"`
}

func main() {
	// 读取源文件
	sourceData, err := os.ReadFile("api/datalist/1背景.json")
	if err != nil {
		fmt.Printf("读取文件失败: %v\n", err)
		return
	}

	// 解析源JSON
	var sourceMaterials []SourceMaterial
	if err := json.Unmarshal(sourceData, &sourceMaterials); err != nil {
		fmt.Printf("解析源JSON失败: %v\n", err)
		return
	}

	// 转换为目标结构
	var targetMaterials []TargetMaterial
	for _, src := range sourceMaterials {
		target := TargetMaterial{
			ID:          src.ID,
			Name:        src.Attributes.Name,
			Desc:        stringValue(src.Attributes.Desc),
			CreatedAt:   src.Attributes.CreatedAt,
			UpdatedAt:   src.Attributes.UpdatedAt,
			PublishedAt: src.Attributes.PublishedAt,
		}

		// 复制图片信息
		target.Img.Data.ID = strconv.Itoa(src.Attributes.Img.Data.ID)
		target.Img.Data.Name = src.Attributes.Img.Data.Attributes.Name
		target.Img.Data.Width = src.Attributes.Img.Data.Attributes.Width
		target.Img.Data.Height = src.Attributes.Img.Data.Attributes.Height
		target.Img.Data.URL = src.Attributes.Img.Data.Attributes.URL
		target.Img.Data.PreviewURL = src.Attributes.Img.Data.Attributes.PreviewURL
		target.Img.Data.Formats = src.Attributes.Img.Data.Attributes.Formats

		targetMaterials = append(targetMaterials, target)
	}

	// 转换为JSON并格式化
	outputData, err := json.MarshalIndent(targetMaterials, "", "  ")
	if err != nil {
		fmt.Printf("生成JSON失败: %v\n", err)
		return
	}

	// 写入新文件
	err = os.WriteFile("api/datalist/converted_背景.json", outputData, 0644)
	if err != nil {
		fmt.Printf("写入文件失败: %v\n", err)
		return
	}

	fmt.Println("转换完成！输出文件: api/datalist/converted_背景.json")
}

func stringValue(s *string) string {
	if s == nil {
		return ""
	}
	return *s
}
